@@ -45,6 +45,7 @@
 #include "parse.h"
 #include "audit.h"
 #include "output.h"
+#include "xml.h"
 
 /****************************************************************************
  * parse_argb_color - should be used for all your color parsing needs
@@ -467,7 +468,7 @@ const char *parse_argb_color( const char *color, CARD32 *pargb )
 		}else if( color[0] == 's' || color[0] == 'S' )
 		{
 			if( mystrncasecmp( &color[1], "aturation(", 10) == 0 )
-				return parse_hsv_part( &color[11], 2, pargb );
+				return parse_hsv_part( &color[11], 2, pargb ); 
 			else if( mystrncasecmp( &color[1], "at(", 3) == 0 )
 				return parse_hsv_part( &color[4], 2, pargb );
 		}else if( color[0] == 'v' || color[0] == 'V' )
@@ -522,6 +523,8 @@ const char *parse_argb_color( const char *color, CARD32 *pargb )
 						free( ptr );
 				}
 			}
+			if( !success && strlen(color) < 30 )
+				show_warning( "unable to parse color \"%s\"", color );
 			return success?&(color[i]):color;
 		}
 	}
@@ -553,24 +556,25 @@ const char *parse_hue( const char *color, int *hue )
 /****************************************************************************
  * Some usefull parsing functions
  ****************************************************************************/
-char         *
-find_doublequotes (char *ptr)
+char*
+find_doublequotes ( const char *ptr)
 {
-	if (ptr)
-	{
-		if (*ptr == '"')
-			ptr++;
+	register int i = 0 ;
+	if (ptr == NULL)
+		return NULL;
 
-		if (*ptr != '"')
-		{
-			while ((ptr = strchr (ptr, '"')))
-				if (*(ptr - 1) == '\\')
-					ptr++;
-				else
-					break;
-		}
+	if (ptr[i] == '"')
+		++i;
+
+	while(ptr[i] != '"')
+	{
+		if( ptr[i] == '\\' ) 
+			++i ;
+		++i ;
+		if( ptr[i] == '\0' )
+		   	return NULL;
 	}
-	return ptr;
+	return (char*)&ptr[i];
 }
 
 /****************************************************************************
@@ -581,8 +585,10 @@ find_doublequotes (char *ptr)
 char         *
 stripcpy (const char *source)
 {
-	const char   *ptr;
+	const char   *ptr = NULL;
 
+	if( source == NULL ) 
+		return NULL;
 	for (; isspace ((int)*source); source++);
 	for (ptr = source + strlen (source); ptr > source && isspace ((int)*(ptr - 1)); ptr--);
 	return mystrndup (source, ptr - source);
@@ -599,7 +605,7 @@ stripcpy (const char *source)
 char         *
 stripcpy2 (const char *source, int tab_sensitive)
 {
-	char         *ptr = (char *)source, *tail;
+	const char *ptr = source, *tail;
 
 	if (*ptr != '"')
 		ptr = find_doublequotes (ptr);
@@ -615,43 +621,54 @@ stripcpy2 (const char *source, int tab_sensitive)
 
 /* here we'll strip comments and skip whitespaces */
 char         *
-stripcomments (char *source)
+stripcomments2 (char *source, char **comments )
 {
-	register char *ptr;
+	int start = 0, curr = 0 ;
 
 	/* remove comments from the line */
-	while (isspace ((int)*source))
-		source++;
-	for (ptr = (char *)source; *ptr; ptr++)
+	while (isspace ((int)source[start])) ++start;
+	
+	for (curr = start; source[curr]; curr++)
 	{
-		if (*ptr == '"')
+		if (source[curr] == '"')
 		{
-			if ((ptr = find_doublequotes (ptr)) == NULL)
+			const char *ptr = find_doublequotes (&source[curr]) ;
+			if ( ptr == NULL )
 			{
-				ptr = source + strlen (source);
+				while(source[++curr]); 
+				break;
+			}else
+				curr = (int)(ptr - source) ;
+		}
+		if (source[curr] == '#')
+		{									   /* checking if it is not a hex color code */
+			int           i = curr;
+
+			while(isxdigit ((int)source[++i]));
+
+			if (i-curr < 4 || i-curr > 13 || (source[i] && !isspace ((int)source[i])))
+			{
+				if( comments ) 
+					*comments = mystrdup( &source[curr+1] );
 				break;
 			}
-		}
-		if (*ptr == '#')
-		{									   /* checking if it is not a hex color code */
-			int           i;
-
-			for (i = 1; isxdigit ((int)*(ptr + i)); i++);
-
-			if (i < 4 || i > 13 || (*(ptr + i) && !isspace ((int)*(ptr + i))))
-			{
-				for (ptr--; ptr > source && isspace ((int)*ptr); ptr--);
-				*(ptr + 1) = '\0';
-				/* we'll exit the loop automagically */
-			} else
-				ptr += i - 1;
+			
+			curr = i - 1;
 		}
 	}
-	for (ptr--; isspace ((int)*ptr) && ptr > source; ptr--);
-	*(ptr + 1) = '\0';
-	return source;
+	do{ --curr ; }while( isspace ((int)source[curr]) && curr > start);
+	source[curr+1] = '\0';
+	return &source[start];
 }
 
+char         *
+stripcomments (char *source)
+{
+	return stripcomments2 (source, NULL );
+}
+
+
+
 char         *
 strip_whitespace (char *str)
 {
@@ -672,45 +689,89 @@ strip_whitespace (char *str)
  * Strips leading and trailing whitespace
  *
  ****************************************************************************/
+char *
+tokenskip( const char *ptr, unsigned int n_tokens )
+{
+    int    tok_no = 0 ;  ;
+    register int  curr = 0 ;
+
+    if( ptr == NULL ) return NULL ;
+    while( ++tok_no <= n_tokens && ptr[curr] != '\0' )
+	{
+		while (!isspace ((int)ptr[curr]) && ptr[curr])
+        {   /* we have to match doublequotes if we encounter any, to allow for spaces in filenames */
+			if (ptr[curr] == '"')
+			{
+				register const char *closing_quote = find_doublequotes (&ptr[curr]);
+				if (closing_quote)
+					curr = (int)(closing_quote-ptr);
+			}
+			++curr;
+		}
+
+		while (isspace ((int)ptr[curr])) ++curr;
+	}
+    return (char*)&ptr[curr];
+}
+
+/****************************************************************************
+ * Copies a whitespace separated token into a new, malloc'ed string
+ * Strips leading and trailing whitespace
+ * returns pointer to the end of token in source string
+ ****************************************************************************/
+char         *
+parse_token (const char *source, char **trg)
+{
+	int tok_start = 0, tok_end;
+	char *next_tok_ptr ; 
+	while( isspace(source[tok_start]) ) ++tok_start;
+	next_tok_ptr = tokenskip( &source[tok_start], 1 );
+	tok_end = (int)(next_tok_ptr-source) ; 
+	if( tok_end > tok_start )
+		if( source[tok_end] != '\0' )  /* then it points to beginning of the next token */ 	
+			do{	--tok_end;}while( tok_end > tok_start && isspace(source[tok_end-1]));
+	
+	if( trg ) 
+		*trg = mystrndup (&source[tok_start], tok_end - tok_start );
+	
+	return next_tok_ptr;
+}
+
 
 char         *
 tokencpy (const char *source)
 {
-	const char   *ptr;
-
-	for (; isspace ((int)*source); source++);
-	for (ptr = source; !isspace ((int)*ptr) && *ptr; ptr++);
-	return mystrndup (source, ptr - source);
+	char *token = NULL ; 
+	parse_token (source, &token);	
+	return token;
 }
 
-char *
-tokenskip( char *start, unsigned int n_tokens )
+char         *
+parse_tab_token (const char *source, char **trg)
 {
-    register int    i ;
-    register char   *cur = start ;
+	char         *ptr;
 
-    if( cur == NULL ) return cur ;
-    for (i = 0; i < n_tokens; i++)
-	{
-		while (!isspace ((int)*cur) && *cur)
-        {   /* we have to match doublequotes if we encounter any, to allow for spaces in filenames */
-			if (*cur == '"')
-			{
-				register char *ptr = find_doublequotes (cur);
+	for (; isspace ((int)*source); source++);
+	for (ptr = (char *)source; *ptr != '\t' && *ptr; ptr++);
+	*trg = mystrndup (source, ptr - source);
+	return ptr;
+}
 
-				if (ptr)
-					while (cur != ptr)
-						cur++;
-			}
-			cur++;
-		}
-
-		while (isspace ((int)*cur) && *cur)
-			cur++;
-		if (*cur == '\0')
-			break;
+char         *
+parse_token_strip_quotes (const char *source, char **trg)
+{
+	char *token = NULL ; 
+	char *next_token = parse_token (source, trg?&token:NULL);	
+	if( trg ) 
+	{	
+		if( token && token[0] == '\"' )
+		{	
+			*trg = stripcpy2 (token, 0);
+			free( token );
+		}else
+			*trg = token ;
 	}
-    return cur;
+	return next_token;
 }
 
 /****************************************************************************
@@ -756,49 +817,6 @@ quotestr (char *dest, const char *src, int maxlen)
 	return maxlen - n;
 }
 
-/****************************************************************************
- *
- * Copies a whitespace separated token into a new, malloc'ed string
- * Strips leading and trailing whitespace
- * returns pointer to the end of token in source string
- ****************************************************************************/
-
-char         *
-parse_token (const char *source, char **trg)
-{
-	char         *ptr;
-
-	for (; isspace ((int)*source); source++);
-	for (ptr = (char *)source; !isspace ((int)*ptr) && *ptr; ptr++);
-	*trg = mystrndup (source, ptr - source);
-	return ptr;
-}
-
-char         *
-parse_tab_token (const char *source, char **trg)
-{
-	char         *ptr;
-
-	for (; isspace ((int)*source); source++);
-	for (ptr = (char *)source; *ptr != '\t' && *ptr; ptr++);
-	*trg = mystrndup (source, ptr - source);
-	return ptr;
-}
-
-char         *
-parse_filename (const char *source, char **trg)
-{
-	for (; isspace ((int)*source); source++);
-
-	if (*source == '"')
-	{
-		if ((*trg = stripcpy2 (source, 0)) != NULL)
-			source += strlen (*trg) + 2;
-	} else
-		source = parse_token (source, trg);
-	return (char *)source;
-}
-
 /* used in ParseMouse and in communication with modules */
 char         *
 parse_signed_int (register char *tline, int *val_return, int *sign_return)
@@ -849,6 +867,75 @@ parse_func_args (char *tline, char *unit, int *func_val)
 	return tline[0]?tline+1:tline;
 }
 
+/* Math expression parsing algorithm. */
+double parse_math(const char* str, char** endptr, double size) {
+	double total = 0;
+	char op = '+';
+	char minus = 0;
+	char logical_not = 0;
+	const char* startptr = str;
+	if( str == NULL ) 
+		return 0 ;
+
+	while (isspace((int)*str)) str++;
+	if( *str == '!' ) 
+	{
+		logical_not = 1;
+		++str ;
+	}else if( *str == '-' ) 
+	{
+		minus = 1 ;
+		++str ;
+	}
+
+	while (*str) 
+	{
+		while (isspace((int)*str)) str++;
+		if (!op) 
+		{
+			if (*str == '+' || *str == '-' || *str == '*' || *str == '/') op = *str++;
+			else if (*str == '-') { minus = 1; str++; }
+			else if (*str == '!') { logical_not = 1; str++; }
+			else if (*str == ')') { str++; break; }
+			else break;
+		} else 
+		{
+			char* ptr;
+			double num;
+			
+			if (*str == '(') 
+				num = parse_math(str + 1, &ptr, size);
+            else if (*str == '$') 
+			{
+            	for (ptr = (char*)str + 1 ; *ptr && !isspace(*ptr) && *ptr != '+' && *ptr != '-' && *ptr != '*' && *ptr != '!' && *ptr != '/' && *ptr != ')' ; ptr++);
+               	num = asxml_var_nget((char*)str + 1, ptr - (str + 1));
+            }else 
+				num = strtod(str, &ptr);
+			
+			if (str != ptr) 
+			{
+				if (*ptr == '%') num *= size / 100.0, ptr++;
+				if (minus) num = -num;
+				if (logical_not) num = !num;
+				
+				if (op == '+') total += num;
+				else if (op == '-') total -= num;
+				else if (op == '*') total *= num;
+				else if (op == '/' && num) total /= num;
+			} else 
+				break;
+			str = ptr;
+			op = '\0';
+			minus = logical_not = 0;
+		}
+	}
+	if (endptr) *endptr = (char*)str;
+	show_debug(__FILE__,"parse_math",__LINE__,"Parsed math [%s] with reference [%.2f] into number [%.2f].", startptr, size, total);
+	return total;
+}
+
+
+
 char         *
 parse_geometry (register char *tline,
                 int *x_return, int *y_return,
@@ -1001,14 +1088,13 @@ scan_for_hotkey (char *txt)
 }
 
 
-/* Conversion of the coma separated values into a list of strings */
 char         *
-get_comma_item (char *ptr, char **item_start, char **item_end)
+get_string_list_item (char *ptr, char **item_start, char **item_end, char separator)
 {
 
 	if (ptr == NULL)
 		return NULL;
-	while (*ptr && (isspace ((int)*ptr) || *ptr == ','))
+	while (*ptr && (isspace ((int)*ptr) || *ptr == separator))
 		ptr++;
 	if (*ptr == '\0')
 		return NULL;
@@ -1019,11 +1105,11 @@ get_comma_item (char *ptr, char **item_start, char **item_end)
 		if ((*item_end = find_doublequotes (ptr)) == NULL)
 			return NULL;
 		ptr = *item_end;
-		while (*ptr && !(isspace ((int)*ptr) || *ptr == ','))
+		while (*ptr && !(isspace ((int)*ptr) || *ptr == separator))
 			ptr++;
 	} else
 	{
-		while (*ptr && *ptr != ',')
+		while (*ptr && *ptr != separator)
 			ptr++;
 		*item_end = ptr;
 	}
@@ -1031,23 +1117,27 @@ get_comma_item (char *ptr, char **item_start, char **item_end)
 }
 
 char        **
-comma_string2list (char *string)
+compound_string2string_list (char *string, char separator, Bool duplicate, int *nitems_return)
 {
 	register char *ptr;
 	int           items;
 	char        **list = NULL, *item_start = NULL, *item_end = NULL;
 
-	if (string == NULL)
+	if (string == NULL )
 		return NULL;
 
 	ptr = string;
 	items = 0;
 	while (*ptr)
 	{
-		if ((ptr = get_comma_item (ptr, &item_start, &item_end)) == NULL)
+		if ((ptr = get_string_list_item (ptr, &item_start, &item_end, separator)) == NULL)
 			break;
 		items++;
 	}
+	
+	if( nitems_return ) 
+		*nitems_return = items ;
+
 	if (items > 0)
 	{
 		register int  i;
@@ -1057,16 +1147,24 @@ comma_string2list (char *string)
 		ptr = string;
 		for (i = 0; i < items; i++)
 		{
-			if ((ptr = get_comma_item (ptr, &item_start, &item_end)) == NULL)
+			if ((ptr = get_string_list_item (ptr, &item_start, &item_end, separator)) == NULL)
 				break;
-			list[i] = mystrndup (item_start, (item_end - item_start));
+			if( duplicate ) 
+				list[i] = mystrndup (item_start, (item_end - item_start));
+			else
+			{	
+				list[i] = item_start ; 
+				if( ptr == item_end && *ptr != '\0' )
+					++ptr ; 
+				*item_end = '\0';
+			}
 		}
 	}
 	return list;
 }
 
 char         *
-list2comma_string (char **list)
+string_list2compound_string (char **list, char separator)
 {
 	char         *string = NULL;
 
@@ -1093,7 +1191,7 @@ list2comma_string (char **list)
 					{
 						*(dst++) = *(src++);
 					}
-					*(dst++) = ',';
+					*(dst++) = separator;
 				}
 			}
 			*(dst - 1) = '\0';
@@ -1102,6 +1200,28 @@ list2comma_string (char **list)
 	return string;
 }
 
+
+
+
+/* Conversion of the coma separated values into a list of strings */
+char         *
+get_comma_item (char *ptr, char **item_start, char **item_end)
+{
+	return get_string_list_item (ptr, item_start, item_end, ',');
+}
+
+char        **
+comma_string2list (char *string)
+{
+	return compound_string2string_list (string, ',', True, NULL);
+}
+
+char         *
+list2comma_string (char **list)
+{
+	return string_list2compound_string (list, ',');
+}
+
 void
 destroy_string_list( char **list )
 {
@@ -1161,3 +1281,31 @@ make_tricky_text( char *src )
     return trg;
 }
 
+char *interpret_ascii_string( const char *str )
+{
+	int src_i = 0, dst_i = 0; 
+	char *dst = safemalloc( (str?strlen(str):0)+1 );
+	
+	while( str[src_i] != '\0' ) 
+	{
+		if( str[src_i] == '\\' )
+		{
+			++src_i ; 
+			if( str[src_i] == 't' ) 
+				dst[dst_i++] = '\t' ;
+			else if( str[src_i] == 'n' ) 
+				dst[dst_i++] = '\n' ;
+			else if( str[src_i] == 'r' ) 
+				dst[dst_i++] = '\r' ;
+			else if( str[src_i] == '\\' ) 
+				dst[dst_i++] = '\\' ;
+			else if( str[src_i] == '\0' ) 
+				break;
+		}else
+			dst[dst_i++] = str[src_i] ;
+		
+		++src_i ;
+	}
+	dst[dst_i] = '\0' ;
+	return dst;
+}
