@@ -442,26 +442,66 @@ clip_root_pixmap( Pixmap root_pixmap, int width, int height )
 	return result;
 }	 
 
-
-ASImage      *
-mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int height, int flip )
+static ASImage      *
+mystyle_make_image_int (MyStyle * style, int root_x, int root_y, int crop_x, int crop_y, int crop_width, int crop_height, int scale_width, int scale_height, int flip, ASImage *underlayment, int overlay_type )
 {
 	ASImage      *im = NULL;
 	Pixmap        root_pixmap;
-	Bool transparency = False ;
+	Bool transparency = TransparentMS(style) ;
 	int preflip_width, preflip_height ;
+	int width, height ; 
+	
+	if(underlayment ) 
+	{
+		if( overlay_type >= TEXTURE_SCALED_TRANSPIXMAP )
+			overlay_type = TEXTURE_TRANSPIXMAP + (overlay_type - TEXTURE_SCALED_TRANSPIXMAP);
+		if( overlay_type < TEXTURE_TRANSPIXMAP || overlay_type >= TEXTURE_TRANSPIXMAP_END ) 
+			overlay_type = TEXTURE_TRANSPIXMAP_ALPHA ;
+		
+		if( transparency )
+			transparency = (overlay_type != TEXTURE_TRANSPIXMAP_ALPHA);
+	} 
 
+	if( crop_x < 0 ) 
+	{
+		root_x += crop_x ; 
+		width = scale_width-crop_x ; 
+		crop_x = 0 ;
+		if( width < crop_width )
+			width = crop_width;
+	}else
+	{
+		width = crop_width + crop_x ; 
+		if( width < scale_width ) 
+			width = scale_width ;
+	}
+
+	if( crop_y < 0 ) 
+	{
+		root_y += crop_y ; 
+		height = scale_height-crop_y ; 
+		crop_y = 0 ;
+		if( height < crop_height )
+			height = crop_height;
+	}else
+	{
+		height = crop_height + crop_y ; 
+		if( height < scale_height ) 
+			height = scale_height ;
+	}
+	
 	if (width < 1)
 		width = 1;
 	if (height < 1)
 		height = 1;
+		
+		
     LOCAL_DEBUG_OUT ("style \"%s\", texture_type = %d, im = %p, tint = 0x%lX, geom=(%dx%d%+d%+d), flip = %d", style->name, style->texture_type,
                      style->back_icon.image, style->tint, width, height, root_x, root_y, flip);
 
-	transparency = TransparentMS(style);
-
-	if(  transparency )
+	if( transparency )
 	{	/* in this case we need valid root image : */
+		
 		if (ASDefaultScr->RootImage == NULL)
 		{
 			unsigned int  root_w, root_h;
@@ -660,9 +700,104 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 		im->back_color = style->colors.back ;
 		show_warning( "MyStyle \"%s\" : failed to generate background image - using solid fill instead with color #0x%8.8X", style->name, style->colors.back );
 	}
+
+	if( underlayment )
+	{
+		 ASImageLayer  layers[2];
+		 int           index = 1;      /* default is alphablending !!! */
+
+		 init_image_layers (&layers[0], 2);
+
+		 layers[0].im = underlayment;
+		 index = overlay_type - TEXTURE_TRANSPIXMAP;
+
+		 LOCAL_DEBUG_OUT ("index = %d", index);
+		 layers[0].merge_scanlines = mystyle_merge_scanlines_func_xref[index];
+         layers[0].dst_x = 0;
+         layers[0].dst_y = 0;
+         layers[0].clip_x = 0;
+         layers[0].clip_y = 0;
+		 layers[0].clip_width = width;
+		 layers[0].clip_height = height;
+
+		 layers[1].im = im;
+		 layers[1].merge_scanlines = layers[0].merge_scanlines;
+		 layers[1].dst_x = 0;
+		 layers[1].dst_y = 0;
+		 layers[1].clip_x = 0;
+		 layers[1].clip_y = 0;
+		 layers[1].clip_width = width;
+		 layers[1].clip_height = height;
+
+		{
+			ASImage *tmp = merge_layers(ASDefaultVisual, &layers[0], 2, 
+										width, height, 
+										ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+			if( tmp )
+			{
+				if( im )
+				{
+					if (style->texture_type == TEXTURE_SHAPED_SCALED_PIXMAP ||
+						style->texture_type == TEXTURE_SHAPED_PIXMAP)
+					{
+						/*we need to keep alpha channel intact */
+						LOCAL_DEBUG_OUT( "copying alpha channel from %p to %p", im, tmp );
+						copy_asimage_channel(tmp, IC_ALPHA, im, IC_ALPHA);
+					}
+					safe_asimage_destroy (im);
+				}
+				im = tmp ;
+			}
+		}
+	}
+
+	if( style->overlay ) 
+	{
+		ASImage *overlayed = mystyle_make_image_int (style->overlay, root_x, root_y, crop_x, crop_y, crop_width, crop_height, scale_width, scale_height, flip, im, style->overlay_type );
+/* 		fprintf( stderr, "overlay_style = %p\n", style->overlay ); */
+		if( overlayed ) 
+		{
+			if( im && overlayed != im ) 
+				safe_asimage_destroy (im);
+			im = overlayed ;
+		}			
+	}
+	if( crop_x > 0 || crop_y > 0 || 
+		(crop_width > 0 && crop_width < im->width ) ||
+		(crop_height > 0 && crop_height < im->height ) ) 
+	{
+		ASImage *cropped_im = tile_asimage( ASDefaultVisual, im,
+											crop_x, crop_y,
+											crop_width, crop_height,
+											TINT_LEAVE_SAME,
+											ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+		if( cropped_im ) 
+		{
+			if( im && cropped_im != im ) 
+				safe_asimage_destroy (im);
+			im = cropped_im ;
+		}			
+	}
 	return im;
 }
 
+ASImage      *
+mystyle_make_image(MyStyle * style, int root_x, int root_y, int width, int height, int flip )
+{
+	if( style != NULL ) 
+		return mystyle_make_image_int (style, root_x, root_y, 0, 0, width, height, width, height, flip, NULL, 0 );
+	return NULL ;
+}
+
+ASImage      *
+mystyle_crop_image(MyStyle * style, int root_x, int root_y, int crop_x, int crop_y, int width, int height, int scale_width, int scale_height, int flip )
+{
+	if( style != NULL ) 
+		return mystyle_make_image_int (style, root_x, root_y, crop_x, crop_y, width, height, scale_width, scale_height, flip, NULL, 0 );
+	return NULL ;
+}
+
+
 /*************************************************************************/
 /* Mystyle creation/deletion                                             */
 /*************************************************************************/
@@ -888,6 +1023,8 @@ mystyle_merge_font( MyStyle *style, MyFont *font, Bool override, Bool copy)
 void
 mystyle_merge_styles (MyStyle * parent, MyStyle * child, Bool override, Bool copy)
 {
+	if( parent == NULL || child == NULL ) 
+		return;
 	if (parent->set_flags & F_FONT)
         mystyle_merge_font( child, &(parent->font), override, copy);
 
@@ -1063,6 +1200,25 @@ mystyle_merge_styles (MyStyle * parent, MyStyle * child, Bool override, Bool cop
 			}
 		}
 	}
+	if (parent->set_flags & F_OVERLAY)
+	{
+		if ((override == True) || !(child->set_flags & F_OVERLAY))
+		{
+			child->flags &= ~F_OVERLAY;
+			child->flags |= parent->flags & F_OVERLAY;
+			child->overlay = parent->overlay ;
+			child->overlay_type = parent->overlay_type ;
+			if (copy == False)
+			{
+				child->user_flags &= ~F_OVERLAY;
+				child->inherit_flags |= F_OVERLAY;
+			} else
+			{
+				child->user_flags |= F_OVERLAY;
+				child->inherit_flags &= ~F_OVERLAY;
+			}
+		}
+	}
 	child->set_flags = child->user_flags | child->inherit_flags;
 }
 
@@ -1119,6 +1275,8 @@ void
 mystyle_merge_colors (MyStyle * style, int type, char *fore, char *back,
 					   char *gradient, char *pixmap)
 {
+	if( style == NULL ) 
+		return ;
 	if ((fore != NULL) && !((*style).user_flags & F_FORECOLOR))
 	{
 		if (parse_argb_color (fore, &((*style).colors.fore)) != fore)
@@ -1215,7 +1373,8 @@ mystyle_inherit_font (MyStyle * style, MyFont * font)
 ASImageBevel *
 mystyle_make_bevel (MyStyle * style, ASImageBevel * bevel, int hilite, Bool reverse)
 {
-	if (style && hilite != 0)
+	if (style && (hilite&HILITE_MASK) != 0 && 
+		(hilite&(NO_HILITE_INLINE|NO_HILITE_OUTLINE)) != (NO_HILITE_INLINE|NO_HILITE_OUTLINE))
 	{
         int extra_hilite = get_flags (hilite, EXTRA_HILITE)?2:0;
 		if (bevel == NULL)
@@ -1250,8 +1409,7 @@ mystyle_make_bevel (MyStyle * style, ASImageBevel * bevel, int hilite, Bool reve
                 bevel->left_inline = bevel->top_inline = bevel->right_inline = bevel->bottom_inline = extra_hilite;
 #endif
             }
-        }else if( get_flags( hilite, NO_HILITE_INLINE ) )
-			clear_flags( hilite, NO_HILITE_INLINE );
+        }
 
 #endif
         if (get_flags (hilite, LEFT_HILITE))
