@@ -78,6 +78,20 @@ struct ASImageManager *get_screen_image_manager(ScreenInfo *scr)
 	return scr->image_manager;	
 }	 
 
+struct ASFontManager *get_screen_font_manager(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->font_manager;	
+}	 
+
+struct ASVisual *get_screen_visual(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->asv;	
+}	 
+
 
 ScreenInfo *get_current_screen()
 {
@@ -233,23 +247,36 @@ ASErrorHandler (Display * dpy, XErrorEvent * event)
 }
 
 
-int
-ConnectX (ScreenInfo * scr, unsigned long event_mask)
+void 
+init_ScreenInfo(ScreenInfo *scr) 
 {
-    
+	if( scr )
+	{
+		memset (scr, 0x00, sizeof (ScreenInfo));
+		scr->Look.magic = MAGIC_MYLOOK ;
+	}
+}
+
+int
+ConnectXDisplay (Display *display, ScreenInfo * scr, Bool as_manager)
+{
+    int width_mm, height_mm ;
+	int display_dpcmx = 0, display_dpcmy = 0;
+	int button_w, button_h, mini_w, mini_h ;
+	
+	if( display == NULL ) 
+		return -1;
+
+	dpy = display ;
+	
 	if( scr == NULL ) 
 	{
 		if( ASDefaultScr == NULL ) 
 			ASDefaultScr = safecalloc( 1, sizeof(ScreenInfo));
 		scr = ASDefaultScr ;
 	}
-	/* Initialize X connection */
-	
-    if (!(dpy = XOpenDisplay (MyArgs.display_name)))
-	{
-        show_error("Can't open display \"%s\". Exiting!", XDisplayName (MyArgs.display_name));
-		exit (1);
-	}
+
+	init_ScreenInfo(scr);	
 
 	x_fd = XConnectionNumber (dpy);
 
@@ -269,8 +296,6 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
     intern_hint_atoms ();
     intern_wmprop_atoms ();
 
-	memset (scr, 0x00, sizeof (ScreenInfo));
-
 	scr->screen = DefaultScreen (dpy);
 	scr->Root = RootWindow (dpy, scr->screen);
 	if (scr->Root == None)
@@ -286,12 +311,34 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
 	asxml_var_insert("xroot.width", scr->MyDisplayWidth);
     asxml_var_insert("xroot.height", scr->MyDisplayHeight);
 
+	width_mm = DisplayWidthMM(dpy, scr->screen);
+	height_mm = DisplayHeightMM(dpy, scr->screen);
+	asxml_var_insert("xroot.widthmm", width_mm);
+    asxml_var_insert("xroot.heightmm", height_mm);
+	
+	display_dpcmx = (scr->MyDisplayWidth * 10 )/ width_mm;
+	display_dpcmy = (scr->MyDisplayHeight * 10 )/ height_mm;
+
+	button_w = (display_dpcmx < 50)?48:64;
+	button_h = (display_dpcmy < 50)?48:64;
+	mini_w = (display_dpcmx <= 44)?max(display_dpcmx/2,12):display_dpcmx-20;
+	mini_h = (display_dpcmy <= 44)?max(display_dpcmy/2,12):display_dpcmy-20;
+	asxml_var_insert(ASXMLVAR_IconButtonWidth, button_w);
+	asxml_var_insert(ASXMLVAR_IconButtonHeight, button_h);
+	asxml_var_insert(ASXMLVAR_IconWidth, (button_w*3)/4);
+	asxml_var_insert(ASXMLVAR_IconHeight, (button_h*3)/4);
+	asxml_var_insert(ASXMLVAR_MinipixmapWidth, mini_w);
+	asxml_var_insert(ASXMLVAR_MinipixmapHeight, mini_h);
+	asxml_var_insert(ASXMLVAR_TitleFontSize, (mini_h*7)/12);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, (mini_h*7)/12+1);
+	asxml_var_insert("font.size", ((mini_h*7)/12));
+
 	scr->CurrentDesk = -1;
 
     scr->RootClipArea.width = scr->MyDisplayWidth;
     scr->RootClipArea.height = scr->MyDisplayHeight;
 
-    if( (scr->wmprops = setup_wmprops( scr, (event_mask&SubstructureRedirectMask), 0xFFFFFFFF, NULL )) == NULL )
+    if( (scr->wmprops = setup_wmprops( scr, as_manager, 0xFFFFFFFF, NULL )) == NULL )
         return -1;
 
     scr->asv = create_asvisual (dpy, scr->screen, DefaultDepth(dpy,scr->screen), NULL);
@@ -319,13 +366,26 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
 	scr->ShmCompletionEventType = XShmGetEventBase(dpy) + ShmCompletion;
 #endif /* SHAPE */
 
-    XSelectInput (dpy, scr->Root, event_mask);
-
 	init_screen_gcs(scr);
 
 	return x_fd;
 }
 
+int
+ConnectX (ScreenInfo * scr, unsigned long event_mask)
+{
+    if (!(dpy = XOpenDisplay (MyArgs.display_name)))
+	{
+        show_error("Can't open display \"%s\". Exiting!", XDisplayName (MyArgs.display_name));
+		exit (1);
+	}
+
+	ConnectXDisplay (dpy, scr, (event_mask&SubstructureRedirectMask));
+    XSelectInput (dpy, scr->Root, event_mask);
+	return x_fd;
+}
+
+
 /***********************************************************************
  *  Procedure:
  *  SetupEnvironment - setup environment variables DISPLAY and HOSTDISPLAY
