@@ -90,6 +90,9 @@ typedef struct {
 
 ASWinListState WinListState = { 0, NULL, None, NULL, NULL };
 
+
+#define WINLIST_BACK_URGENT		BACK_DEFAULT
+
 /**********************************************************************/
 /**********************************************************************/
 /* Our configuration options :                                        */
@@ -104,6 +107,7 @@ WinListConfig *Config = NULL ;
 
 /**********************************************************************/
 
+void retrieve_winlist_astbar_props();
 void SetWinListLook();
 void GetBaseOptions (const char *filename);
 void GetOptions (const char *filename);
@@ -138,19 +142,24 @@ main( int argc, char **argv )
                       WINDOW_NAME_MASK |
                       M_END_WINDOWLIST, 0) < 0 ) 
 		exit(1);               /* no AfterStep */
-    balloon_init (False);
-    Config = CreateWinListConfig ();
 
     /* Request a list of all windows, while we load our config */
     SendInfo ("Send_WindowList", 0);
 
-    LoadBaseConfig ( GetBaseOptions);
+/*  LoadBaseConfig ( GetBaseOptions); */
+	ReloadASEnvironment( NULL, NULL, NULL, False, True );
+
 	LoadColorScheme();
-	LoadConfig ("winlist", GetOptions);
+//	ASModuleConfig *asm_config = parse_asmodule_config_all( WinListConfigClass );
+	Config = AS_WINLIST_CONFIG(parse_asmodule_config_all( WinListConfigClass )/*asm_config*/);
+	
+/* 	LoadConfig ("winlist", GetOptions); */
 	CheckWinListConfigSanity(Config, &(MyArgs.geometry), MyArgs.gravity);
 	ReloadASDatabase();
 	ReloadCategories(True);
     
+	retrieve_winlist_astbar_props();
+		
 	SetWinListLook();
 
     WinListState.main_window = make_winlist_window();
@@ -200,11 +209,15 @@ DeadPipe (int nonsense)
         destroy_ascanvas( &WinListState.main_canvas );
     if( WinListState.main_window )
         XDestroyWindow( dpy, WinListState.main_window );
-    
-	FreeMyAppResources();
+
+	destroy_astbar_props( &(WinListState.tbar_props) );
+	    
+	DestroyCategories();
     
 	if( Config )
-        DestroyWinListConfig(Config);
+        destroy_ASModule_config( AS_MODULE_CONFIG(Config));
+
+	FreeMyAppResources();
 
 #ifdef DEBUG_ALLOCS
     print_unfreed_mem ();
@@ -232,7 +245,6 @@ SetWinListLook()
 	default_winlist_style[0] = '*' ;
 	strcpy( &(default_winlist_style[1]), MyName );
 
-	retrieve_winlist_astbar_props();
     mystyle_get_property (Scr.wmprops);
     for( i = 0 ; i < BACK_STYLES ; ++i )
 		Scr.Look.MSWindow[i] = NULL ;
@@ -243,23 +255,27 @@ SetWinListLook()
 	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_FOCUSED, Config->FocusedStyle?Config->FocusedStyle:"(null)", Scr.Look.MSWindow[BACK_FOCUSED] );
     Scr.Look.MSWindow[BACK_STICKY] = mystyle_find( Config->StickyStyle );
 	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_STICKY, Config->StickyStyle?Config->StickyStyle:"(null)", Scr.Look.MSWindow[BACK_STICKY] );
+	Scr.Look.MSWindow[BACK_URGENT] = mystyle_find( Config->UrgentStyle );
+	LOCAL_DEBUG_OUT( "Configured MyStyle %d \"%s\" is %p", BACK_URGENT, Config->UrgentStyle?Config->UrgentStyle:"(null)", Scr.Look.MSWindow[BACK_URGENT] );
 
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
         static char *default_window_style_name[BACK_STYLES] ={"focused_window_style","unfocused_window_style","sticky_window_style", NULL};
         if( Scr.Look.MSWindow[i] == NULL )
             Scr.Look.MSWindow[i] = mystyle_find( default_window_style_name[i] );
-        if( Scr.Look.MSWindow[i] == NULL )
+        if( Scr.Look.MSWindow[i] == NULL && i != BACK_URGENT )
             Scr.Look.MSWindow[i] = mystyle_find_or_default( default_winlist_style );
 		LOCAL_DEBUG_OUT( "MyStyle %d is \"%s\"", i, Scr.Look.MSWindow[i]?Scr.Look.MSWindow[i]->name:"none" );
     }
     free( default_winlist_style );
 	
+	mylook_set_font_size_var (&(Scr.Look));
+	
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
     PrintWinListConfig (Config);
-    Print_balloonConfig ( Config->balloon_conf );
+    Print_balloonConfig ( Config->asmodule_config.balloon_configs[0]);
 #endif
-    balloon_config2look( &(Scr.Look), Config->balloon_conf, "*WinListBalloon" );
+    balloon_config2look( &(Scr.Look), NULL, Config->asmodule_config.balloon_configs[0], "*WinListBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -274,6 +290,7 @@ GetBaseOptions (const char *filename)
 void
 GetOptions (const char *filename)
 {
+#if 0
     START_TIME(option_time);
     WinListConfig *config = ParseWinListOptions( filename, MyName );
 
@@ -283,9 +300,11 @@ GetOptions (const char *filename)
     /* Need to merge new config with what we have already :*/
 	MergeWinListOptions( Config, config );
 
-    if (config->style_defs)
-        ProcessMyStyleDefinitions (&(config->style_defs));
+    if (config->asmodule_config.style_defs)
+        ProcessMyStyleDefinitions (&(config->asmodule_config.style_defs));
+	DestroyWinListConfig( config );
     SHOW_TIME("Config parsing",option_time);
+#endif	
 }
 
 
@@ -398,7 +417,6 @@ DispatchEvent (ASEvent * event)
 			}
 	        break;
 	    case PropertyNotify:
-			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
 			if( event->w == Scr.Root || event->w == Scr.wmprops->selection_window ) 
 			{
 				handle_wmprop_event (Scr.wmprops, &(event->x));
@@ -497,7 +515,9 @@ make_winlist_window()
 	shints.win_gravity = Config->gravity ;
 
 	extwm_hints.pid = getpid();
-	extwm_hints.flags = EXTWM_PID|EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager|EXTWM_TypeDock|EXTWM_TypeASModule ;
+	extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet|EXTWM_StateSet ;
+	extwm_hints.state_flags = EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager ;
+	extwm_hints.type_flags = EXTWM_TypeDock|EXTWM_TypeASModule ;
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
 	set_client_cmd (w);
@@ -929,7 +949,12 @@ configure_tbar_icon( ASTBarData *tbar, ASWindowData *wd )
 
 		if( get_flags( Config->flags, WINLIST_ScaleIconToTextHeight ) )
 		{
-			 height = width = calculate_astbar_height( tbar );
+			if( Config->UseName >= ASN_NameTypes ) /* no name! */
+				height = width = asxml_var_get( ASXMLVAR_TitleFontSize );
+			else
+				height = width = calculate_astbar_height( tbar );
+			if( height < 5 ) 
+				height = width = 16 ; 
 		}	 
 		if( get_flags(Config->set_flags, WINLIST_IconSize) )
 		{
@@ -955,6 +980,42 @@ configure_tbar_icon( ASTBarData *tbar, ASWindowData *wd )
 	}	 
 }
 
+void
+do_blink_urgent_bar( void *vdata )
+{
+	ASTBarData *tbar = (ASTBarData *)vdata ;
+	set_astbar_focused( tbar, WinListState.main_canvas, !IsASTBarFocused(tbar) );
+	timer_new (500, do_blink_urgent_bar, tbar);	
+}
+
+static void 
+focus_winlist_button( ASWindowData *wd )
+{
+	timer_remove_by_data( wd->bar );  /* just in case */ 
+
+	if( wd->focused ) 
+	{
+		set_astbar_style_ptr( wd->bar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[BACK_FOCUSED] );
+		set_astbar_focused( wd->bar, NULL, True );
+        if( WinListState.focused && WinListState.focused != wd )
+		{
+			WinListState.focused->focused = False ; 
+			focus_winlist_button( WinListState.focused );
+		}
+		WinListState.focused = wd ;
+	}else
+	{
+		if( get_flags(wd->state_flags, AS_Urgent) && Scr.Look.MSWindow[BACK_URGENT] != NULL )
+		{
+	    	set_astbar_style_ptr( wd->bar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[BACK_URGENT] );
+			set_astbar_focused( wd->bar, NULL, True );
+			timer_new (500, do_blink_urgent_bar, wd->bar);	
+		}else
+			set_astbar_focused( wd->bar, NULL, False );
+	}
+	if( DoesBarNeedsRendering(wd->bar) )
+		render_astbar( wd->bar, WinListState.main_canvas );		
+}
 
 static void
 configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
@@ -974,6 +1035,7 @@ configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
 				   
     	delete_astbar_tile( tbar, -1 );
 		LOCAL_DEBUG_OUT( "setting bar border to %+d, %+d", tbar->h_border, tbar->v_border );
+
     	set_astbar_style_ptr( tbar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[BACK_FOCUSED] );
 		if( WinListState.tbar_props )
 		{
@@ -1013,12 +1075,12 @@ configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
     	{
         	set_astbar_style_ptr( tbar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_STICKY] );
         	set_astbar_hilite( tbar, BACK_UNFOCUSED, sbevel );
-        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->SCompositionMethod );
+        	set_astbar_composition_method( tbar, BACK_UNFOCUSED, Config->SCompositionMethod );
     	}else
     	{
         	set_astbar_style_ptr( tbar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_UNFOCUSED] );
         	set_astbar_hilite( tbar, BACK_UNFOCUSED, ubevel );
-        	set_astbar_composition_method( tbar, BACK_FOCUSED, Config->UCompositionMethod );
+        	set_astbar_composition_method( tbar, BACK_UNFOCUSED, Config->UCompositionMethod );
     	}
 		
 		if( get_flags( Config->flags, ASWL_ShowIcon ) && (Config->IconLocation%10 == 5 ))
@@ -1042,18 +1104,7 @@ configure_tbar_props( ASTBarData *tbar, ASWindowData *wd, Bool focus_only )
 
 		set_astbar_balloon( tbar, 0, name, encoding );
 	}
-    set_astbar_focused( tbar, WinListState.main_canvas, wd->focused );
-    if( wd->focused )
-	{
-        if( WinListState.focused && WinListState.focused != wd )
-		{
-			WinListState.focused->focused = False ;
-			set_astbar_focused( WinListState.focused->bar, WinListState.main_canvas, False );
-			if( focus_only ) 
-				render_astbar( WinListState.focused->bar, WinListState.main_canvas );
-		}
-		WinListState.focused = wd ;
-    }
+	focus_winlist_button( wd );
 }
 
 void
@@ -1111,6 +1162,9 @@ LOCAL_DEBUG_OUT("tbar = %p, wd = %p", tbar, wd );
     if( WinListState.focused == wd )
 		WinListState.focused = NULL ;
 
+	if( tbar )
+		timer_remove_by_data( tbar );  /* just in case */ 
+
     if( i < WinListState.windows_num  )
     {
         while( ++i < WinListState.windows_num )
