@@ -29,6 +29,8 @@
 
 #include "../../configure.h"
 #include "../../libAfterStep/asapp.h"
+#include "../../libAfterStep/desktop_category.h"
+#include "../../libAfterConf/afterconf.h"
 #include "dirtree.h"
 void          init_func_data (FunctionData * data);
 int           txt2func (const char *text, FunctionData * fdata, int quiet);
@@ -104,6 +106,8 @@ dirtree_delete (dirtree_t * tree)
 	if (tree->minipixmap_extension != NULL)
 		free (tree->minipixmap_extension);
 	free_func_data (&tree->command);
+	if (tree->de != NULL)
+		unref_desktop_entry(tree->de);
 
 	free (tree);
 }
@@ -116,7 +120,7 @@ make_absolute (const char *path1, const char *path2)
 	if( path1 == NULL || path2 == NULL )
 		return mystrdup("./") ;
 	if (*path2 == '/' || *path2 == '~' || *path2 == '$')
-		path = PutHome (path2);
+		path = copy_replace_envvar(path2);
 	else
 		/* relative path */
 	{
@@ -254,11 +258,81 @@ dirtree_set_command (dirtree_t * tree, struct FunctionData *command, int recurse
 	}
 }
 
+void 
+dirtree_add_category (dirtree_t *tree, ASCategoryTree *ct, ASDesktopCategory *dc, Bool include_children, ASHashTable *exclusions)
+{
+	dirtree_t    *t = NULL ; 
+	int i, entries_num ;
+	char **entries ; 
+
+	ASSERT_TREE(tree);
+	ASSERT_TREE(dc);
+	ASSERT_TREE(ct);
+
+	entries_num = PVECTOR_USED(dc->entries);
+	if( entries_num == 0 ) 
+		return;
+	entries = PVECTOR_HEAD(char*, dc->entries );
+	LOCAL_DEBUG_OUT( "DesktopCategory \"%s\", has %d entries", dc->name, entries_num );
+	for( i = 0 ; i < entries_num ; ++i ) 
+	{
+		ASDesktopEntry *de = NULL ; 
+		ASDesktopCategory *sub_dc = NULL ; 
+
+		if( exclusions ) 
+			if( get_hash_item( exclusions, AS_HASHABLE(entries[i]), NULL ) == ASH_Success ) 
+				continue;
+				
+		de = fetch_desktop_entry( ct, entries[i] ); 
+		LOCAL_DEBUG_OUT( "entry \"%s\", de = %p", entries[i], de );
+		if( de == NULL ) 
+			continue;
+		if( de->type == ASDE_TypeDirectory ) 
+		{	
+			if( !include_children ) 
+				continue;
+			sub_dc = fetch_desktop_category( ct, entries[i] );
+			if( sub_dc == NULL || PVECTOR_USED(sub_dc->entries ) == 0 ) 
+				continue;
+		}
+		t = dirtree_new ();
+		t->name = mystrdup (de->Name);
+		t->comment = mystrdup (de->Comment);
+		
+		ref_desktop_entry( de );
+		t->de = de ; 
+		
+		t->parent = tree;
+		t->next = tree->child;
+		tree->child = t;			
+		
+		if( sub_dc ) 
+		{				   
+			t->flags |= DIRTREE_DIR;
+			dirtree_add_category (t, ct, sub_dc, include_children, exclusions);
+		}	 
+	}	
+}
+
+
+void 
+dirtree_add_category_by_name (dirtree_t *tree, const char *cat_name, Bool include_children, ASHashTable *exclusions)
+{
+	ASCategoryTree *ct = CombinedCategories ; 
+	ASDesktopCategory *dc = NULL ;
+	
+	ASSERT_TREE(tree);
+
+	dc = name2desktop_category( cat_name, &ct ); 
+	dirtree_add_category (tree, ct, dc, include_children, exclusions);
+}
+
 int
 dirtree_parse (dirtree_t * tree, const char *file)
 {
 	FILE         *fp;
 	char         *str;
+	ASHashTable  *exclusions = NULL; 
 
 	ASSERT_TREE_INT(tree,1);
 	
@@ -280,6 +354,35 @@ dirtree_parse (dirtree_t * tree, const char *file)
 		/* ignore comments and blank lines */
 		if (*ptr == '#' || *ptr == '\0')
 			continue;
+		if( !mystrncasecmp (ptr, "exclude", 7) )
+		{
+			char *excl_name ; 
+			if( exclusions == NULL ) 
+				exclusions = create_ashash( 0, casestring_hash_value, casestring_compare, string_destroy );
+			if( exclusions ) 
+			{	
+				excl_name = stripcpy2 (ptr +7, 0);
+				add_hash_item( exclusions, AS_HASHABLE(excl_name), NULL );
+			}
+			continue;
+		}		
+		
+		if( !mystrncasecmp (ptr, "category", 8) )
+		{
+			char *cat_name;
+			Bool include_children = False;
+			ptr+= 8 ;
+			if( !mystrncasecmp (ptr, "_tree", 5) )
+			{
+				include_children = True ;
+				ptr += 5 ;	  
+			}
+			cat_name = stripcpy2 (ptr, 0);
+			dirtree_add_category_by_name (tree, cat_name, include_children, exclusions);
+			free( cat_name );
+			continue;
+		}
+		   
 		if( !mystrncasecmp (ptr, "include", 7) )
 		{
 			do_include = True ;
@@ -349,8 +452,13 @@ dirtree_parse (dirtree_t * tree, const char *file)
 			for (tmp = ptr + strlen (ptr); tmp > ptr && isspace (*(tmp - 1)); tmp--);
 			if (tmp != ptr)
 				tree->minipixmap_extension = mystrndup (ptr, tmp - ptr);
-		} else if (!mystrncasecmp (ptr, "minipixmap", 10))
+		} else if (!mystrncasecmp (ptr, "minipixmap", 10) || !mystrncasecmp (ptr, "smallminipixmap", 15))
 		{
+			if( ptr[0] == 's' || ptr[0] == 'S' )
+			{	
+				set_flags(tree->flags, DIRTREE_ICON_IS_SMALL);
+				ptr += 5 ;
+			}
 			for (ptr += 10; isspace (*ptr); ptr++);
 			if (tree->icon != NULL)
 				free (tree->icon);
@@ -373,6 +481,11 @@ dirtree_parse (dirtree_t * tree, const char *file)
 			if (tree->name != NULL)
 				free (tree->name);
 			tree->name = mystrdup (ptr);
+		}else if (!mystrncasecmp (ptr, "Comment", 7))
+		{
+			if (tree->comment != NULL)
+				free (tree->comment);
+			tree->comment = stripcpy2(ptr+7,False);
 		}
 	}
 	free (str);
