@@ -254,6 +254,7 @@ WaitWindowLoop( char *pattern, long timeout )
 	text = safemalloc(64+strlen(pattern)+1);
 	sprintf( text, "Waiting for window matching \"%s\" ... Press button to cancel.", pattern );
 	hint = create_ashint_window( ASDefaultScr, &(Scr.Look), text);
+	free( text );
 	while (!done)
 	{
 		do
@@ -362,7 +363,7 @@ DigestEvent( ASEvent *event )
 			event->client = NULL ;
 		}else
 		{
-			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) != NULL  )
 			{	
 				event->client = NULL ;
 				event->widget = NULL ;	 
@@ -430,7 +431,8 @@ DigestEvent( ASEvent *event )
 	        if (ASWIN_GET_FLAGS(asw, AS_Shaded) && canvas != asw->frame_sides[tbar_side] )
 			{
 				event->context = C_NO_CONTEXT ;				
-				XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
+				if( asw->frame_sides[tbar_side] )
+					XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
 			}else if( w != asw->frame )
             {
                 if( event->w == asw->frame )
@@ -499,6 +501,10 @@ DigestEvent( ASEvent *event )
 								  	(event->x.type == LeaveNotify),
 								  	(last_pointer_root_x != pointer_root_x || last_pointer_root_y != pointer_root_y) );
 		}		
+		if( event->x.type == LeaveNotify ) 
+		{
+			withdraw_active_balloon_from( TitlebarBalloons );
+		}
 		last_pointer_root_x = pointer_root_x ;
 		last_pointer_root_y = pointer_root_y ;
 
@@ -606,7 +612,13 @@ DispatchEvent ( ASEvent *event, Bool deffered )
             * indicates that the warp is done */
             if( get_flags( AfterStepState, ASS_WarpingMode ) )
                 EndWarping();
-            HandleButtonPress (event, deffered);
+			if( event->x.xbutton.button > Button3 )
+			{/* buttons 4 and 5 are for scrollwheel */
+				ASInternalWindow *internal = event->client?event->client->internal:NULL ;
+				if( internal && internal->on_scroll_event )
+					internal->on_scroll_event( internal, event );
+			}else
+		        HandleButtonPress (event, deffered);
             break;
         case ButtonRelease:
             /* if warping, a button press, non-warp keypress, or pointer motion
@@ -902,8 +914,13 @@ HandlePropertyNotify (ASEvent *event)
     }else if( atom == XA_WM_COMMAND || atom == XA_WM_CLIENT_MACHINE )
 	{
 		update_cmd_line_hints (asw->w, atom, asw->hints, asw->status );
+	}else if( atom == _XA_NET_WM_WINDOW_OPACITY )
+	{	
+	 	if( event->context == C_WINDOW && event->w != asw->frame ) 
+			on_window_opacity_changed( asw );
 	}else if( NeedToTrackPropChanges(atom) )
         on_window_hints_changed( asw );
+
     /* we have to do the complete refresh of hints, since we have to override WH_HINTS with database, etc. */
 }
 
@@ -944,16 +961,29 @@ HandleClientMessage (ASEvent *event)
 		ASFlagType extwm_flags = 0, as_flags = 0;
 		CARD32 props[2] ;
 		XClientMessageEvent *xcli = &(event->x.xclient) ;
+		
 		props[0] = xcli->data.l[1] ;
 		props[1] = xcli->data.l[2] ;
-
+		
 		translate_atom_list (&extwm_flags, EXTWM_State, &props[0], 2);
 		/* now we need to translate EXTWM flags into AS flags : */
 		as_flags = extwm_state2as_state_flags( extwm_flags );
 		if( xcli->data.l[0] == EXTWM_StateRemove ) 
+		{
 			as_flags = ASWIN_GET_FLAGS(event->client,as_flags);
-		else if( xcli->data.l[0] == EXTWM_StateAdd ) 
+		}else if( xcli->data.l[0] == EXTWM_StateAdd ) 
 			as_flags = as_flags&(~ASWIN_GET_FLAGS(event->client,as_flags));
+
+		if( props[0] == _XA_NET_WM_STATE_DEMANDS_ATTENTION ||
+			props[1] == _XA_NET_WM_STATE_DEMANDS_ATTENTION  ) 
+		{/* requires special treatment as it competes with ICCCM HintUrgency in WM_HINTS */
+			Bool set = True ; 
+			if( xcli->data.l[0] == EXTWM_StateRemove || 
+				(xcli->data.l[0] == EXTWM_StateToggle && ASWIN_GET_FLAGS(event->client,AS_Urgent)))  
+				set = False ; 
+			set_extwm_urgency_state( event->client->w, set );
+		}		
+		
 		if( as_flags != 0 ) 
 			toggle_aswindow_status(event->client, as_flags );
 	}	 
@@ -1054,7 +1084,7 @@ HandleMapNotify ( ASEvent *event )
     ASWIN_CLEAR_FLAGS(asw, AS_Iconic);
     complete_wm_state_transition( asw, NormalState );
 	LOCAL_DEBUG_OUT( "no_focus = %d, force_activation = %d, AcceptsFocus = %ld", no_focus, force_activation, ASWIN_HFLAGS(asw,AS_AcceptsFocus) );
-	if( !no_focus && ASWIN_HFLAGS(asw,AS_AcceptsFocus) )
+	if( !no_focus && ASWIN_FOCUSABLE(asw) )
     	activate_aswindow (asw, force_activation, False);
     broadcast_config( M_MAP, asw );
     /* finally reaches Normal state */
@@ -1284,7 +1314,7 @@ HandleEnterNotify (ASEvent *event)
     if (asw == NULL )
 		return;
 
-    if (ASWIN_HFLAGS(asw,AS_AcceptsFocus))
+    if (ASWIN_FOCUSABLE(asw))
 	{
         if (!get_flags(Scr.Feel.flags, ClickToFocus) || asw->internal != NULL )
 		{
@@ -1352,17 +1382,24 @@ HandleConfigureRequest ( ASEvent *event )
 	{
         unsigned long xwcm;
         xwcm = cre->value_mask & (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
-		xwc.x = cre->x;
-		xwc.y = cre->y;
-		xwc.width = cre->width;
-		xwc.height = cre->height;
-		xwc.border_width = cre->border_width;
-		LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
-        XConfigureWindow (dpy, event->w, xwcm, &xwc);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for untracked window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.x = cre->x;
+			xwc.y = cre->y;
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			xwc.border_width = cre->border_width;
+			LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
 		return;
 	}
 
-	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) ||
+	if( ( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) && !get_flags(cre->value_mask, CWWidth|CWHeight))||
 		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
@@ -1384,19 +1421,20 @@ HandleConfigureRequest ( ASEvent *event )
     if( cre->value_mask & (CWWidth|CWHeight|CWX|CWY) )
     {
         XRectangle new_anchor = asw->anchor;
-        int grav_x, grav_y ;
-        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
-
 
         if( cre->value_mask&CWWidth )
             new_anchor.width = cre->width ;
         if( cre->value_mask&CWHeight )
             new_anchor.height = cre->height ;
-        if( cre->value_mask&CWX )
-            new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
-        if( cre->value_mask&CWY )
-            new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
-
+		if( !ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+		{
+    	    int grav_x, grav_y ;
+	        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
+        	if( cre->value_mask&CWX )
+            	new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
+        	if( cre->value_mask&CWY )
+            	new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
+		}
 LOCAL_DEBUG_OUT( "old anchor(%dx%d%+d%+d), new_anchor(%dx%d%+d%+d)", asw->anchor.width, asw->anchor.height, asw->anchor.x, asw->anchor.y, new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         validate_window_anchor( asw, &new_anchor );
 LOCAL_DEBUG_OUT( "validated_anchor(%dx%d%+d%+d)", new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
