@@ -96,6 +96,7 @@ void send_window_list_func_handler( FunctionData *data, ASEvent *event, int modu
 void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module );
 void signal_reload_GTKRC_file_handler( FunctionData *data, ASEvent *event, int module );
 void KIPC_send_message_all_handler( FunctionData *data, ASEvent *event, int module );
+void set_func_handler( FunctionData *data, ASEvent *event, int module );
 void test_func_handler( FunctionData *data, ASEvent *event, int module );
 void screenshot_func_handler( FunctionData *data, ASEvent *event, int module );
 void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -211,6 +212,7 @@ void SetupFunctionHandlers()
 
     function_handlers[F_QUICKRESTART]       = quickrestart_func_handler ;
     function_handlers[F_SEND_WINDOW_LIST]   = send_window_list_func_handler ;
+    function_handlers[F_SET]                = set_func_handler ;
     function_handlers[F_Test]               = test_func_handler ;
 
 	function_handlers[F_TAKE_WINDOWSHOT]    =
@@ -510,6 +512,12 @@ void ExecutePendingFunctions()
 	    DoExecuteFunction( sf );
 }
 
+void DestroyPendingFunctionsQueue()
+{
+    if( FunctionQueue )
+		destroy_asbidirlist( &FunctionQueue );
+}
+
 /***********************************************************************
  *  Procedure:
  *	DeferExecution - defer the execution of a function to the
@@ -1012,7 +1020,7 @@ void setlayer_func_handler( FunctionData *data, ASEvent *event, int module )
 void change_desk_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     if( event->client )
-	change_aswindow_desktop( event->client, data->func_val[0], False );
+		change_aswindow_desktop( event->client, data->func_val[0], False );
 }
 
 void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1085,11 +1093,13 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     ASMenu *menu = NULL;
 	char * menu_name = data->text? data->text:data->name ;
-    if( menu_name )
+	LOCAL_DEBUG_OUT( "menu_name = \"%s\", client = %p, internal = %p", menu_name?"NULL":menu_name, event->client, event->client->internal );
+    if( menu_name && menu_name[0] != '\0' )
 		menu = find_asmenu( menu_name );
     else if( event->client && event->client->internal )
     {
 		ASMagic *data = event->client->internal->data ;
+		LOCAL_DEBUG_OUT( "data = %p, magic = %lX", data, data->magic );
 		if( data->magic == MAGIC_ASMENU )
 	    	menu = (ASMenu*)data;
     }
@@ -1696,12 +1706,100 @@ void gethelp_func_handler( FunctionData *data, ASEvent *event, int module )
 
 void wait_func_handler( FunctionData *data, ASEvent *event, int module )
 {
+	ASWindow *asw ;
 	char *complex_pattern = data->text ;
 	if( data->name && data->name[1] == ':' )
 		complex_pattern = &(data->name[2]);
 
-	WaitWindowLoop( complex_pattern, -1 );
-	LOCAL_DEBUG_OUT( "Wait completed for \"%s\"", complex_pattern );
+	asw = WaitWindowLoop( complex_pattern, -1 );
+	LOCAL_DEBUG_OUT( "Wait completed for \"%s\", asw = %p, data->text = \"%s\"", complex_pattern, asw, data->text?data->text:"(null)" );
+	if( asw && data->text ) 
+	{
+		/* 1. parse text into a name_list struct */	
+		name_list *style = string2DatabaseStyle (data->text);
+		LOCAL_DEBUG_OUT( "style = %p, set_data_flags = 0x%lX, set_flags = 0x%lX", style, style?style->set_data_flags:0, style?style->set_flags:0 );
+		if( style ) 
+		{	/* 2. apply set data from name_list to asw */	
+			int new_vx = Scr.Vx ;
+			int	new_vy = Scr.Vy ;
+			int x, y ; 
+			int width, height ;
+			ASFlagType geom_flags = get_flags (style->set_data_flags, STYLE_DEFAULT_GEOMETRY)?style->default_geometry.flags:0;
+			ASFlagType state_flags = 0 ;
+
+			if( get_flags (style->set_data_flags, STYLE_VIEWPORTX) )
+				new_vx = style->ViewportX ;
+			else
+				new_vx = asw->status->viewport_x ;
+			if( get_flags (style->set_data_flags, STYLE_VIEWPORTY) )
+				new_vy = style->ViewportY ;
+			else
+				new_vy = asw->status->viewport_y ;
+			
+			x = get_flags( geom_flags, XValue )?style->default_geometry.x:asw->status->x ; 
+			y = get_flags( geom_flags, YValue )?style->default_geometry.y:asw->status->y ; 
+			width = get_flags( geom_flags, WidthValue )?
+						(style->default_geometry.width+asw->status->frame_size[FR_W]+asw->status->frame_size[FR_E]):
+						asw->status->width ; 
+			height = get_flags( geom_flags, HeightValue )?
+						(style->default_geometry.height+asw->status->frame_size[FR_N]+asw->status->frame_size[FR_S]):
+						asw->status->height ; 
+			
+			if( get_flags (style->set_flags, STYLE_STICKY))
+			{
+				if( (get_flags (style->flags, STYLE_STICKY) && !ASWIN_GET_FLAGS(asw,AS_Sticky)) ||
+					(!get_flags (style->flags, STYLE_STICKY) && ASWIN_GET_FLAGS(asw,AS_Sticky)) )
+					set_flags( state_flags, AS_Sticky );
+			}
+			if( get_flags (style->set_flags, STYLE_FULLSCREEN))
+			{
+				if( (get_flags (style->flags, STYLE_FULLSCREEN) && !ASWIN_GET_FLAGS(asw,AS_Fullscreen)) ||
+					(!get_flags (style->flags, STYLE_FULLSCREEN) && ASWIN_GET_FLAGS(asw,AS_Fullscreen)) )
+					set_flags( state_flags, AS_Fullscreen );
+			}
+
+			if( state_flags != 0 ) 
+				toggle_aswindow_status( asw, state_flags );
+
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+			{	
+			 	if(	get_flags (style->set_data_flags, STYLE_STARTUP_DESK) )
+						change_aswindow_desktop( asw, style->Desk, False );
+			}else
+			{
+				new_vx = Scr.Vx ;	
+				new_vy = Scr.Vy ;
+			}	 
+			
+			LOCAL_DEBUG_OUT( "%s -> viewport : new(%+d%+d)old(%+d%+d) ; geom : new(%dx%d%+d%+d)old(%dx%d%+d%+d)", 
+							 ASWIN_GET_FLAGS(asw,AS_Sticky)?"sticky":"slippery", 
+							 new_vx, new_vy, asw->status->viewport_x, asw->status->viewport_y,
+							 width, height, x, y, asw->status->width, asw->status->height, asw->status->x, asw->status->y );
+			
+							   
+			if( get_flags (style->set_data_flags, STYLE_DEFAULT_GEOMETRY|STYLE_VIEWPORTX|STYLE_VIEWPORTY) )
+			{
+				x += Scr.Vx - asw->status->viewport_x + new_vx ;
+				y += Scr.Vy - asw->status->viewport_y + new_vy ;
+				moveresize_aswindow_wm( asw, x, y, width, height, False );
+			}	 
+			
+			if( get_flags (style->set_data_flags, STYLE_LAYER) && ASWIN_LAYER(asw) != style->layer )
+			{
+				LOCAL_DEBUG_OUT( "new_layer = %d", 	  style->layer );
+				change_aswindow_layer( asw, style->layer );
+			}
+							 
+			if( get_flags (style->set_flags, STYLE_START_ICONIC ) )
+			{
+				if ( (get_flags (style->flags, STYLE_START_ICONIC ) && !ASWIN_GET_FLAGS(asw, AS_Iconic))||  
+					 (!get_flags (style->flags, STYLE_START_ICONIC ) && ASWIN_GET_FLAGS(asw, AS_Iconic)))
+					set_window_wm_state( event->client, get_flags (style->flags, STYLE_START_ICONIC ), False );
+			}	 
+			
+			style_delete (style, NULL);
+		}	
+	}	 
 	XSync (dpy, 0);
 }
 
@@ -1907,6 +2005,54 @@ void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module
  	}	
 }	 
 
+void set_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	if( data->text != NULL ) 
+	{
+		char *eq_ptr = strchr( data->text,'=' ); 
+		if( eq_ptr ) 
+		{
+			int val  = 0 ;
+			char *tail = eq_ptr+1; 
+			val = (int)parse_math(tail, &tail, strlen(tail));
+			LOCAL_DEBUG_OUT( "tail = \"%s\", val = %d", tail, val );
+			if( tail > eq_ptr+1 ) 
+			{
+				*eq_ptr = '\0' ; 
+				asxml_var_insert(data->text, val);
+				if( strncmp( data->text, "menu.", 5 ) == 0 )
+				{
+					ASFlagType change_flag = 0 ; 
+					ASFlagType old_look_flags = Scr.Look.flags ; 
+					if( strcmp( data->text, ASXMLVAR_MenuShowMinipixmaps ) == 0 )
+						change_flag = MenuMiniPixmaps ;
+					else if( strcmp( data->text, ASXMLVAR_MenuShowUnavailable ) == 0 )
+						change_flag = MenuShowUnavailable ;
+					else if( strcmp( data->text, ASXMLVAR_MenuTxtItemsInd ) == 0 )
+						change_flag = TxtrMenuItmInd ;
+					if( change_flag != 0 ) 
+					{
+						if( !val ) 
+							clear_flags( Scr.Look.flags, change_flag );
+						else 
+							set_flags( Scr.Look.flags, change_flag );
+					}else if( strcmp( data->text, ASXMLVAR_MenuRecentSubmenuItems) == 0 )
+					{
+						Scr.Feel.recent_submenu_items = (val <= 0)?0:val ;
+					}
+					
+					if( old_look_flags != Scr.Look.flags ) 
+					{
+						/* need to referesh menus maybe? */
+					}
+					LOCAL_DEBUG_OUT( "old_flags = %lX, flags = %lX", old_look_flags, Scr.Look.flags );
+				}
+				*eq_ptr = '=' ; 
+			}
+		}
+	}
+}
+
 void test_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	 /* add test command processing here : */
