@@ -541,6 +541,7 @@ LOCAL_DEBUG_OUT( "**CONFG Client(%lx(%s))->status(%ux%u%+d%+d,%s,%s(%d>-%d))",
 
         moveresize_canvas(  asw->frame_canvas, asw->status->x, asw->status->y,
                             new_width, new_height );
+/*		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas); */
     }
 	return moved;
 }
@@ -882,6 +883,9 @@ on_window_hints_changed( ASWindow *asw )
     static ASRawHints raw_hints ;
     ASHints *hints = NULL, *old_hints = NULL;
 	Bool tie_changed = False;
+	ASStatusHints scratch_status ; /* all we need from it is AS_Urgent state change */
+	Bool status_changed = False;
+	
 
     if( AS_ASSERT(asw) )
         return ;
@@ -891,8 +895,12 @@ on_window_hints_changed( ASWindow *asw )
         return ;
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_hints( NULL, NULL, &raw_hints );
-    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
-    destroy_raw_hints( &raw_hints, True );
+
+	memset( &scratch_status, 0x00, sizeof( scratch_status ) );
+			
+    hints = merge_hints( &raw_hints, Database, &scratch_status, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
+    
+	destroy_raw_hints( &raw_hints, True );
     if( hints )
     {
         show_debug( __FILE__, __FUNCTION__, __LINE__,  "Window management hints collected and merged for window %X", asw->w );
@@ -922,7 +930,29 @@ on_window_hints_changed( ASWindow *asw )
     /* we do not want to do a complete refresh of decorations -
 	 * we want to do only what is neccessary: */
 
+	if( get_flags( scratch_status.flags, AS_Urgent ) )
+	{
+		if( !get_flags( asw->status->flags, AS_Urgent ) )
+		{
+			set_flags( asw->status->flags, AS_Urgent );
+			status_changed = True ;				
+		}
+	}else if( get_flags( asw->status->flags, AS_Urgent ) )
+	{
+		ASFlagType extwm_state = 0 ;
+		
+		if( !get_extwm_state_flags (asw->w, &extwm_state) )
+			extwm_state = 0 ;/* just in case */
+		if ( !get_flags (extwm_state, EXTWM_StateDemandsAttention))
+		{
+			clear_flags( asw->status->flags, AS_Urgent );
+			status_changed = True ;				
+		}
+	}
 	if( hints2decorations( asw, old_hints ) )
+		status_changed = True ;
+	
+	if( status_changed) 
 	    on_window_status_changed( asw, False, True );
 
 	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
@@ -936,7 +966,41 @@ on_window_hints_changed( ASWindow *asw )
 	if( mystrcmp( old_hints->icon_name, hints->icon_name ) != 0 )
 	    broadcast_icon_name( asw );
 
-	destroy_hints (old_hints, True);
+	destroy_hints (old_hints, False);
+}
+
+void 
+on_window_opacity_changed( ASWindow *asw )
+{
+    CARD32 new_opacity = NET_WM_WINDOW_OPACITY_OPAQUE ;
+	Bool set = False ;
+	ASDatabaseRecord db_rec ; 
+	
+	if( AS_ASSERT(asw) )
+        return ;
+    if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+		return;
+	set = read_32bit_property (asw->w, _XA_NET_WM_WINDOW_OPACITY, &new_opacity);
+	if ( set && new_opacity == asw->hints->window_opacity && ASWIN_HFLAGS(asw, AS_WindowOpacity) )
+		return;
+	
+	if( fill_asdb_record (Database, asw->hints->names, &db_rec, False ) )
+	{
+		if( get_flags( db_rec.set_data_flags, STYLE_WINDOW_OPACITY ) )
+		{	
+			new_opacity = set_hints_window_opacity_percent( NULL, db_rec.window_opacity);		   
+			set = True ;
+		}
+	}	 
+
+	if( !set )
+		XDeleteProperty( dpy, asw->frame, _XA_NET_WM_WINDOW_OPACITY	);
+	else if( new_opacity != asw->hints->window_opacity || !ASWIN_HFLAGS(asw, AS_WindowOpacity) )
+	{
+		set_flags( asw->hints->flags, AS_WindowOpacity);
+		asw->hints->window_opacity = new_opacity ;
+		set_32bit_property (asw->frame, _XA_NET_WM_WINDOW_OPACITY, XA_CARDINAL, asw->hints->window_opacity );
+	}	 
 }
 
 int
@@ -1016,6 +1080,8 @@ on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured
     if( AS_ASSERT(asw) )
         return ;
 
+/*	get_extwm_state_flags (asw->w, &i); */
+	
 LOCAL_DEBUG_CALLER_OUT( "(%p,%s Update display,%s Reconfigured)", asw, update_display?"":"Don't", reconfigured?"":"Not" );
 LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->status->height, asw->status->x, asw->status->y );
     if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
@@ -1282,7 +1348,7 @@ save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 void
 moveresize_aswindow_wm( ASWindow *asw, int x, int y, unsigned int width, unsigned int height, Bool save_anchor )
 {
-	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, x, y, width, height, save_anchor );
+	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, width, height, x, y, save_anchor );
     if( !AS_ASSERT(asw) )
     {
         ASStatusHints scratch_status  = *(asw->status);
@@ -1368,11 +1434,23 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 		t->status->viewport_x = Scr.Vx ;
 		t->status->viewport_y = Scr.Vy ;
 	}
-	if (!get_flags (t->status->flags, AS_Sticky) && 
-		!ASWIN_HFLAGS(t, AS_UseCurrentViewport))
+	if (!get_flags (t->status->flags, AS_Sticky) )
 	{
-		t->status->x -= t->status->viewport_x;
-		t->status->y -= t->status->viewport_y;
+		Bool absolute_origin = (!ASWIN_HFLAGS(t, AS_UseCurrentViewport));
+		
+		if( absolute_origin && 
+			get_flags( t->hints->flags, AS_Transient ) &&
+			get_flags( t->status->flags, AS_StartPositionUser ))
+		{/* most likely stupid KDE or GNOME app that is abusing USPosition 
+			for no good reason - place it on current viewport */
+			absolute_origin = ( t->status->x >= Scr.MyDisplayWidth || 
+								t->status->y >= Scr.MyDisplayHeight ) ;
+		}
+		if( absolute_origin )
+		{	
+			t->status->x -= t->status->viewport_x;
+			t->status->y -= t->status->viewport_y;
+		}
 	}
 	LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
 	
@@ -1458,11 +1536,12 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 
     	}else if( get_flags(Scr.Feel.flags, NoPPosition))
 		{ 
-      		if( !get_flags( t->hints->flags, AS_Transient ) && 
-		    	!get_flags( status->flags, AS_StartPositionUser ) )
+		
+      		if( !get_flags( t->hints->flags, AS_Transient ) &&
+				!get_flags( t->status->flags, AS_StartPositionUser ))
 	        	clear_flags( t->status->flags, AS_Position );
 		}
-		 if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
+		if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
         	pending_placement = True ;
 		else  if( !get_flags( t->status->flags, AS_Position ))
     	{
@@ -1646,9 +1725,9 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
                 }
             }
         }
+	    on_window_status_changed( asw, True, False );
     }
 
-    on_window_status_changed( asw, True, False );
     if( !get_flags(asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
         broadcast_config(M_CONFIGURE_WINDOW, asw );
 
